<?php
if ( ! class_exists( 'IPQuiz_Db' )) {

    /**
     * Class for manage plugin database
     */
    class IPQuiz_Db
    {
        /**
         * Take db version
         *
         * @var string
         */
        public $version = '1.0';
        
        /**
         * Class construct
         *
         * @return void
         */
        public function __construct()
        {
            global $ipquiz_db_version;
            $ipquiz_db_version = $this->version;
        }
        
        /**
         * Table question
         *
         * @return string
         */
        public function queryQuestion()
        {
            global $wpdb;
            
            $table_name = $wpdb->prefix . 'ipquiz_question';
            $sql = "CREATE TABLE $table_name (
                        qtn_id          int(11)             NOT NULL AUTO_INCREMENT,
                        post_id         int(11)             NOT NULL,
                        qtn_content     text                NOT NULL,
                        qtn_image       int(11)   DEFAULT 0 NOT NULL,
                        qtn_status      smallint  DEFAULT 0 NOT NULL,
                        qtn_type        varchar(20)         NOT NULL,
                        qtn_order       int(11)   DEFAULT 0 NOT NULL,
                    PRIMARY KEY  question_id (qtn_id),
                    KEY post_id (post_id)
            );";

            return $sql;
        }
        
        /**
         * Table answer
         *
         * @return string
         */
        public function queryAnswer()
        {
            global $wpdb;
            
            $table_name = $wpdb->prefix . 'ipquiz_answer';
            $sql = "CREATE TABLE $table_name (
                        ans_id          int(11)             NOT NULL AUTO_INCREMENT,
                        qtn_id          int(11)             NOT NULL,
                        ans_content     text                NOT NULL,
                        ans_status      smallint  DEFAULT 0 NOT NULL,
                        ans_order       int(11)   DEFAULT 0 NOT NULL,
                    PRIMARY KEY  answer_id (ans_id),
                    KEY question_id (qtn_id)
            );";

            return $sql;
        }
        
        /**
         * Table choice
         *
         * @return string
         */
        public function queryChoice()
        {
            global $wpdb;
            
            $table_name = $wpdb->prefix . 'ipquiz_choice';
            $sql = "CREATE TABLE $table_name (
                        cho_id          int(11)             NOT NULL AUTO_INCREMENT,
                        parc_id         int(11)             NOT NULL,
                        quiz_id         int(11)             NOT NULL,
                        qtn_id          int(11)             NOT NULL,
                        cho_date        datetime  DEFAULT '0000-00-00 00:00:00' NOT NULL,
                        cho_time        int(11)   DEFAULT 0 NOT NULL,
                        cho_ip          varchar(64)         NOT NULL,
                        cho_browser     varchar(255)        NOT NULL,
                        cho_status      tinyint             NOT NULL,
                    PRIMARY KEY  choice_id (cho_id),
                    KEY quiz_id (quiz_id),
                    KEY question_id (qtn_id),
                    KEY participant_id (parc_id)
            );";

            return $sql;
        }
        
        /**
         * Table choice answer
         *
         * @return string
         */
        public function queryChoiceAnswer()
        {
            global $wpdb;
            
            $table_name = $wpdb->prefix . 'ipquiz_choice_answer';
            $sql = "CREATE TABLE $table_name (
                        cho_id          int(11)             NOT NULL AUTO_INCREMENT,
                        qtn_id          int(11)             NOT NULL,
                        ans_id          int(11)             NOT NULL,
                    PRIMARY KEY  choice_answer_id (cho_id, ans_id),
                    KEY question_id (qtn_id)
            );";

            return $sql;
        }
        
        /**
         * Table participant
         *
         * @return string
         */
        public function queryParticipant()
        {
            global $wpdb;
            
            $table_name = $wpdb->prefix . 'ipquiz_participant';
            $sql = "CREATE TABLE $table_name (
                        parc_id         int(11)             NOT NULL AUTO_INCREMENT,
                        facebook_id     varchar(255)       NOT NULL,
                        name            varchar(255)        NOT NULL,
                        email           varchar(255)        NOT NULL,
                        link            varchar(255)        NOT NULL,
                        join_date       datetime            DEFAULT '0000-00-00 00:00:00' NOT NULL,
                        meta            text                NOT NULL,
                        point           int(11)             NOT NULL,
                    PRIMARY KEY  participant_id (parc_id)
            );";

            return $sql;
        }
        
        /**
         * Table progress
         *
         * @return string
         */
        public function queryProgress()
        {
            global $wpdb;
            
            $table_name = $wpdb->prefix . 'ipquiz_progress';
            $sql = "CREATE TABLE $table_name (
                        parc_id         int(11)             NOT NULL AUTO_INCREMENT,
                        quiz_id         int(11)             NOT NULL,
                        pro_update      datetime            DEFAULT '0000-00-00 00:00:00' NOT NULL,
                        pro_point       int(11)             NOT NULL,
                        pro_step        tinyint             NOT NULL,
                        pro_status      tinyint             NOT NULL,
                    PRIMARY KEY  progress_id (parc_id,quiz_id)
            );";

            return $sql;
        }
        
        /**
         * Database query
         *
         * @return string
         */
        public function query()
        {
            $sql  = '';
            $sql .= $this->queryQuestion();
            $sql .= $this->queryAnswer();
            $sql .= $this->queryChoice();
            $sql .= $this->queryChoiceAnswer();
            $sql .= $this->queryParticipant();
            $sql .= $this->queryProgress();
            return $sql;
        }
        
        /**
         * install
         *
         * @return void
         */
        public function install()
        {
            global $wpdb, $ipquiz_db_version;
            $installed_ver = get_option( "ipquiz_db_version" );
            
            if( ! $installed_ver || ( $installed_ver != $ipquiz_db_version ) ) {

                $sql = $this->query();

                require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
                dbDelta( $sql );

                update_option( "ipquiz_db_version", $ipquiz_db_version );
            }
        }
        
        /**
         * update
         *
         * @return void
         */
        public function update()
        {
            global $ipquiz_db_version;
            if ( get_site_option( 'ipquiz_db_version' ) != $ipquiz_db_version ) {
                $this->install();
            }
        }
    }

}
?>