<?php
if ( ! class_exists( 'IPQuiz_Model' )) {

    /**
     * Class for handling administrator page
     */
    class IPQuiz_Model
    {
        /**
         * Get Questions
         *
         * @return array
         */
        public function getQuestions( $quizId )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'ipquiz_question';
            $sql   = "
                SELECT * FROM $table WHERE post_id = %d ORDER BY qtn_order
            ";
            $quizId  = absint( $quizId );
            $query   = $wpdb->prepare( $sql, $quizId );
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get Question
         *
         * @param  int $id
         * @return array
         */
        public function getQuestion( $id )
        {
            global $wpdb;
            $id    = absint( $id );
            $table = $wpdb->prefix . 'ipquiz_question';
            $sql   = "
                SELECT * FROM $table WHERE qtn_id = %d
            ";
            $query   = $wpdb->prepare( $sql, $id );
            $results = $wpdb->get_row( $query );
            return $results;
        }
        
        /**
         * Get Answers
         *
         * @return array
         */
        public function getAnswers( $qtnId )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'ipquiz_answer';
            $sql   = "
                SELECT * FROM $table WHERE qtn_id = %d ORDER BY ans_order
            ";
            $qtnId   = absint( $qtnId );
            $query   = $wpdb->prepare( $sql, $qtnId );
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get Answer
         *
         * @param  int $id
         * @return array
         */
        public function getAnswer( $id )
        {
            global $wpdb;
            $id    = absint( $id );
            $table = $wpdb->prefix . 'ipquiz_answer';
            $sql   = "
                SELECT * FROM $table WHERE ans_id = %d
            ";
            $query   = $wpdb->prepare( $sql, $id );
            $results = $wpdb->get_row( $query );
            return $results;
        }
        
        /**
         * Get Answer Order
         *
         * @param  int $qtnId
         * @return array
         */
        public function getAnswerOrder( $qtnId )
        {
            global $wpdb;
            $qtnId = absint( $qtnId );
            $table = $wpdb->prefix . 'ipquiz_answer';
            $sql   = "
                SELECT * FROM $table WHERE qtn_id = %d
            ";
            $query   = $wpdb->prepare( $sql, $qtnId );
            $results = $wpdb->get_results( $query );
            $order   = 1;
            if ( ! empty( $results )) {
                $order = count( $results ) + 1;
            }
            return $order;
        }
        
        /**
         * Get Choices
         *
         * @param  int $quizId
         * @param  int $parcId
         * @return array
         */
        public function getChoices( $quizId, $parcId )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipquiz_choice';
            $table2 = $wpdb->prefix . 'ipquiz_question';
            $query = "
                SELECT * FROM $table
                JOIN $table2 ON $table.qtn_id = $table2.qtn_id
                WHERE 
                    $table.quiz_id = $quizId AND
                    $table.parc_id = $parcId
                ORDER BY $table2.qtn_order
            ";
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get Choice
         *
         * @param  int $id
         * @return array
         */
        public function getChoice( $id )
        {
            global $wpdb;
            $id    = absint( $id );
            $table = $wpdb->prefix . 'ipquiz_choice';
            $sql   = "
                SELECT * FROM $table WHERE cho_id = %d
            ";
            $query   = $wpdb->prepare( $sql, $id );
            $results = $wpdb->get_row( $query );
            return $results;
        }
        
        /**
         * Get Choice by question
         *
         * @param  int $qtnId
         * @return array
         */
        public function getChoiceByQuestion( $qtnId )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'ipquiz_choice';
            $sql   = "
                SELECT * FROM $table WHERE qtn_id = %d ORDER BY cho_date
            ";
            $qtnId   = absint( $qtnId );
            $query   = $wpdb->prepare( $sql, $qtnId );
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Is choice ?
         *
         * @param  int $qtnId
         * @return array
         */
        public function isChoice( $qtnId )
        {
            global $wpdb;
            $ip    = $_SERVER['REMOTE_ADDR'];
            $table = $wpdb->prefix . 'ipquiz_choice';
            $sql   = "
                SELECT * FROM $table WHERE qtn_id = %d AND cho_ip = %d ORDER BY cho_date
            ";
            $qtnId   = absint( $qtnId );
            $query   = $wpdb->prepare( $sql, $qtnId, $ip );
            $results = $wpdb->get_results( $query );
            if ( empty( $results )) {
                return false;
            }
            return true;
        }
        
        /**
         * Get Participants
         *
         * @return array
         */
        public function getParticipants()
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipquiz_participant';
            $query = "
                SELECT * FROM $table
            ";
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get quiz participants
         *
         * @param  int $quizId 
         * @return array
         */
        public function getQuizParticipants( $quizId )
        {
            global $wpdb;
            $table1 = $wpdb->prefix . 'ipquiz_progress';
            $table2 = $wpdb->prefix . 'ipquiz_participant';
            $query = "
                SELECT * FROM $table1
                JOIN $table2 ON $table1.parc_id = $table2.parc_id
                WHERE 
                    $table1.quiz_id = $quizId
                GROUP BY $table1.parc_id
                ORDER BY $table1.pro_point
            ";
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get Participant
         *
         * @param  int $id
         * @return array
         */
        public function getParticipant( $id )
        {
            global $wpdb;
            $id    = absint( $id );
            $table = $wpdb->prefix . 'ipquiz_participant';
            $sql   = "
                SELECT * FROM $table WHERE parc_id = %d
            ";
            $query   = $wpdb->prepare( $sql, $id );
            $results = $wpdb->get_row( $query );
            return $results;
        }
        
        /**
         * Get Participation
         *
         * @param  int $parcId
         * @return array
         */
        public function getParticipation( $parcId )
        {
            global $wpdb;
            $id    = absint( $parcId );
            $table = $wpdb->prefix . 'ipquiz_progress';
            $sql   = "
                SELECT parc_id, COUNT(quiz_id) AS total 
                FROM $table 
                WHERE parc_id = %d 
                GROUP BY parc_id
            ";
            $query   = $wpdb->prepare( $sql, $id );
            $results = $wpdb->get_row( $query );
            return $results;
        }
        
        /**
         * Get statistic data
         *
         * @return array
         */
        public function getStatisticData( $start = null, $end = null )
        {
            global $wpdb;
            
            $table = $wpdb->prefix . 'ipquiz_progress';
            $query = "
                SELECT 
                    DATE_FORMAT(pro_update, '%Y-%m-%d') AS gdate,
                    COUNT(parc_id) AS total_parc, 
                    COUNT(quiz_id) AS total_quiz
                FROM $table";
            
            if ( ! empty( $start ) && ! empty( $end )) 
            {
                if ( $start == $end ) {
                    $query .= " WHERE DATE_FORMAT( pro_update, '%Y-%m-%d' ) = '$start'";
                } else {
                    $query .= " WHERE pro_update BETWEEN '$start' AND '$end'";
                }
            }
            elseif ( ! empty( $start )) {
                $query .= " WHERE DATE_FORMAT( pro_update, '%Y-%m-%d' ) = '$start'";
            }
            elseif ( ! empty( $end )) {
                $query .= " WHERE DATE_FORMAT( pro_update, '%Y-%m-%d' ) = '$end'";
            }
            $query .= " GROUP BY gdate";
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get FB User
         *
         * @param  int $id
         * @return array
         */
        public function getFBUser( $id )
        {
            global $wpdb;
            $id    = absint( $id );
            $table = $wpdb->prefix . 'ipquiz_participant';
            $sql   = "
                SELECT * FROM $table WHERE facebook_id = $id
            ";
            $results = $wpdb->get_row( $sql );
            return $results;
        }
        
        /**
         * Get progress
         *
         * @param  int $quizId
         * @param  int $parcId
         * @return array
         */
        public function getProgress( $quizId, $parcId )
        {
            global $wpdb;
            $id    = absint( $id );
            $table = $wpdb->prefix . 'ipquiz_progress';
            $sql   = "
                SELECT * FROM $table 
                WHERE 
                    quiz_id = $quizId AND parc_id = $parcId
            ";
            $results = $wpdb->get_row( $sql );
            return $results;
        }
        
        /**
         * Insert Question
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insertQuestion( $data, $format = array() )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipquiz_question';
            $data   = (array)$data;
            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }
        
        /**
         * Update Question
         *
         * @param  int   $id
         * @param  array $data
         * @return void
         */
        public function updateQuestion( $id, $data )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipquiz_question';
            $data   = (array)$data;
            $where  = array(
                'qtn_id' => absint( $id )
            );
            $result = $wpdb->update( $table, $data, $where );
            return $result;
        }
        
        /**
         * Delete Question
         *
         * @param  int   $id
         * @return void
         */
        public function deleteQuestion( $id )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipquiz_question';
            $where  = array(
                'qtn_id' => absint( $id )
            );
            
            $result = $wpdb->delete( $table, $where );
            return $result;
        }
        
        /**
         * Insert Answer
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insertAnswer( $data, $format = array() )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipquiz_answer';
            $data   = (array)$data;
            
            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }
        
        /**
         * Update Answer
         *
         * @param  int   $id
         * @param  array $data
         * @return void
         */
        public function updateAnswer( $id, $data )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipquiz_answer';
            $data   = (array)$data;
            $where  = array(
                'ans_id' => absint( $id )
            );
            $result = $wpdb->update( $table, $data, $where );
            return $result;
        }
        
        /**
         * Delete Answer
         *
         * @param  int   $id
         * @return void
         */
        public function deleteAnswer( $id )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipquiz_answer';
            $where  = array(
                'ans_id' => absint( $id )
            );
            $result = $wpdb->delete( $table, $where );
            return $result;
        }
        
        /**
         * Delete Answers
         *
         * @param  int   $qtnId
         * @return void
         */
        public function deleteAnswers( $qtnId )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipquiz_answer';
            $where  = array(
                'qtn_id' => absint( $qtnId )
            );
            $result = $wpdb->delete( $table, $where );
            return $result;
        }
        
        /**
         * Insert Choice
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insertChoice( $data, $format = array() )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipquiz_choice';
            $data   = (array)$data;
            
            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }
        
        /**
         * Update Choice
         *
         * @param  int   $id
         * @param  array $data
         * @return void
         */
        public function updateChoice( $id, $data )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipquiz_choice';
            $data   = (array)$data;
            $where  = array(
                'cho_id' => absint( $id )
            );
            $result = $wpdb->update( $table, $data, $where );
            return $result;
        }
        
        /**
         * Delete Choice
         *
         * @param  int   $id
         * @return void
         */
        public function deleteChoice( $id )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipquiz_choice';
            $where  = array(
                'cho_id' => absint( $id )
            );
            $result = $wpdb->delete( $table, $where );
            return $result;
        }
        
        /**
         * Insert Choice answer
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insertChoiceAnswer( $data, $format = array() )
        {
            global $wpdb;
            
            $table  = $wpdb->prefix . 'ipquiz_choice_answer';
            $data   = (array)$data;
            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }
        
        /**
         * Delete Choice answer
         *
         * @param  int   $choId
         * @return void
         */
        public function deleteChoiceAnswer( $choId )
        {
            global $wpdb;
            
            $table  = $wpdb->prefix . 'ipquiz_choice_answer';
            $where  = array(
                'cho_id' => absint( $choId )
            );
            $result = $wpdb->delete( $table, $where );
            return $result;
        }
        
        /**
         * Insert participant
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insertParticipant( $data, $format = array() )
        {
            global $wpdb;
            
            $table  = $wpdb->prefix . 'ipquiz_participant';
            $data   = (array)$data;
            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }
        
        /**
         * Update participant
         *
         * @param  int   $id
         * @param  array $data
         * @return void
         */
        public function updateParticipant( $id, $data )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipquiz_participant';
            $data   = (array)$data;
            $where  = array(
                'parc_id' => absint( $id )
            );
            $result = $wpdb->update( $table, $data, $where );
            return $result;
        }
        
        /**
         * Insert progress
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insertProgress( $data, $format = array() )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipquiz_progress';
            $data   = (array)$data;
            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }
        
        /**
         * Update progress
         *
         * @param  int   $quizId
         * @param  int   $parcId
         * @param  array $data
         * @return void
         */
        public function updateProgress( $quizId, $parcId, $data )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipquiz_progress';
            $data   = (array)$data;
            $where  = array(
                'quiz_id' => absint( $quizId ),
                'parc_id' => absint( $parcId )
            );
            $result = $wpdb->update( $table, $data, $where );
            return $result;
        }
        
        /**
         * Save question data
         *
         * @param  int $quizId
         * @return void
         */
        public function saveQuestion( $quizId )
        {
            $result = true;
            $quizId = absint( $quizId );
            if ( empty( $_POST['question_data'] )) {
                return $result;
            }
            $quests  = array();
            if ( $current = $this->getQuestions( $quizId ) ){
                foreach( $current as $qtn ) {
                    $qtnid = $qtn->qtn_id;
                    $quests[$qtnid] = $qtn;
                }
            }
            $order = 1;
            $data  = json_decode( stripslashes($_POST['question_data'] ), true );
            foreach( $data as $item )
            {
                if ( empty( $item )) {
                    continue;
                }
                $itemId   = 0;
                $question = false;
                if ( ! empty( $item['id'] )) {
                    $itemId   = absint( $item['id'] );
                    $question = $this->getQuestion( $itemId );
                }
                if ( ! $question ) {
                    $itemId = $this->addQuestion( $quizId, $item, $order );
                } else {
                    if ( array_key_exists( $itemId, $quests )) {
                        unset( $quests[$itemId] );
                    }
                    $this->changeQuestion( $itemId, $item, $order );
                }
                if ( ! empty( $item['answer'] )) {
                    $this->saveAnswer( $itemId, $item['answer'] );
                }
                $order++;
            }
            if ( ! empty( $quests )) {
                foreach( $quests as $id => $qtn ) 
                {
                    $this->deleteAnswers( $id );
                    $this->deleteQuestion( $id );
                }
            }
            return $result;
        }
        
        /**
         * Add question
         *
         * @param  int   $itemId
         * @param  array $data
         * @return void
         */
        public function addQuestion( $quizId, $data, $order = 1 )
        {
            $content  = '';
            if ( ! empty( $data['question'] )) {
                $content = $data['question'];
            }
            $image = 0;
            if ( ! empty( $data['imageid'] )) {
                $image = absint( $data['imageid'] );
            }
            $type = 'single';
            if ( ! empty( $data['type'] )) {
                $type = apply_filters( 'pre_term_name', $data['type'] );
            }
            $toInsert = array(
                'post_id'       => $quizId,
                'qtn_content'   => $content,
                'qtn_image'     => $image,
                'qtn_type'      => $type,
                'qtn_status'    => 1,
                'qtn_order'     => $order
            );
            $result = $this->insertQuestion( $toInsert );
            return $result;
        }
        
        /**
         * Change question
         *
         * @param  int   $itemId
         * @param  array $data
         * @param  int   $order
         * @return void
         */
        public function changeQuestion( $itemId, $data, $order = 1 )
        {
            $toUpdate = array(
                'qtn_order' => $order
            );
            if ( ! empty( $data['question'] )) {
                $toUpdate['qtn_content'] = $data['question'];
            }
            if ( ! empty( $data['imageid'] )) {
                $toUpdate['qtn_image'] = absint( $data['imageid'] );
            }
            if ( ! empty( $data['type'] )) {
                $toUpdate['qtn_type'] = apply_filters( 'pre_term_name', $data['type'] );
            }
            $result = $this->updateQuestion( $itemId, $toUpdate );
            return $result;
        }
        
        /**
         * Save answer data
         *
         * @param  int $questId
         * @param  array $data
         * @return void
         */
        public function saveAnswer( $questId, $data )
        {
            $result  = true;
            $order   = 1;
            $answers = array();
            if ( $current = $this->getAnswers( $questId ) ){
                foreach( $current as $ans ) {
                    $ansid = $ans->ans_id;
                    $answers[$ansid] = $ans;
                }
            }
            foreach( $data as $item )
            {
                if ( empty( $item )) {
                    continue;
                }
                $itemId = 0;
                $answer = false;
                if ( ! empty( $item['id'] )) {
                    $itemId = absint( $item['id'] );
                    $answer = $this->getAnswer( $itemId );
                }
                if ( ! $answer ) {
                    $itemId = $this->addAnswer( $questId, $item, $order );
                } else {
                    if ( array_key_exists( $itemId, $answers )) {
                        unset( $answers[$itemId] );
                    }
                    $this->changeAnswer( $itemId, $item, $order );
                }
                $order++;
            }
            if ( ! empty( $answers )) {
                foreach( $answers as $id => $ans ) 
                {
                    $this->deleteAnswer( $id );
                }
            }
            return $result;
        }
        
        /**
         * Add answer
         *
         * @param  int   $questId
         * @param  array $data
         * @param  int   $order
         * @return void
         */
        public function addAnswer( $questId, $data, $order = 1 )
        {
            $content  = '';
            if ( ! empty( $data['answer'] )) {
                $content = $data['answer'];
            }
            $status = 1;
            if ( ! empty( $data['status'] )) {
                $status = absint( $data['status'] );
            }
            $toInsert = array(
                'qtn_id'        => $questId,
                'ans_content'   => $content,
                'ans_status'    => $status,
                'ans_order'     => $order
            );
            $result = $this->insertAnswer( $toInsert );
            return $result;
        }
        
        /**
         * Change answer
         *
         * @param  int   $itemId
         * @param  array $data
         * @param  int   $order
         * @return void
         */
        public function changeAnswer( $itemId, $data, $order = 1 )
        {
            $toUpdate = array(
                'ans_order' => $order
            );
            if ( ! empty( $data['answer'] )) {
                $toUpdate['ans_content'] = $data['answer'];
            }
            if ( ! empty( $data['status'] )) {
                $toUpdate['ans_status'] = absint( $data['status'] );
            }
            $result = $this->updateAnswer( $itemId, $toUpdate );
            return $result;
        }
        
        /**
         * Save setting data
         *
         * @param  int $quizId
         * @return void
         */
        public function saveQuizSettings( $quizId )
        {
            $quizId  = absint( $quizId );
            $data    = $this->_getSettingData();
            $setting = new IPQuiz_Setting( $quizId, '__ipquiz-item-setting' );
            $result  = $setting->setSettings( $data );
            return $result;
        }
        
        /**
         * Get setting data
         *
         * @return array
         */
        protected function _getSettingData()
        {
            $setting = array();
            $data    = array_merge( $_POST, $_GET );
            
            //Content
            if ( ! empty( $data['content_image'] )) {
                $setting['image'] = absint( $data['content_image'] );
            }
            if ( ! empty( $data['login_info'] )) {
                $setting['login_info'] = $data['login_info'];
            }
            if ( ! empty( $data['point_info'] )) {
                $setting['point_info'] = $data['point_info'];
            }
            if ( ! empty( $data['congrats_info'] )) {
                $setting['congrats_info'] = $data['congrats_info'];
            }
            if ( ! empty( $data['timeout_info'] )) {
                $setting['timeout_info'] = $data['timeout_info'];
            }
            
            //Setting
            if ( ! empty( $data['timelimit'] )) {
                $setting['timelimit'] = absint( $data['timelimit'] );
            }
            if ( ! empty( $data['correct_point'] )) {
                $setting['correct_point'] = absint( $data['correct_point'] );
            }
            if ( ! empty( $data['wrong_point'] )) {
                $setting['wrong_point'] = absint( $data['wrong_point'] );
            }
            if ( ! empty( $data['design'] )) {
                $setting['design'] = apply_filters( 'pre_term_name', $data['design'] );
            }
            if ( ! empty( $data['style'] )) {
                $setting['style'] = apply_filters( 'pre_term_name', $data['style'] );
            }
            if ( ! empty( $data['correct_audio'] )) {
                $setting['correct_audio'] = absint( $data['correct_audio'] );
            }
            if ( ! empty( $data['wrong_audio'] )) {
                $setting['wrong_audio'] = absint( $data['wrong_audio'] );
            }
            if ( ! empty( $data['effect'] )) {
                $setting['effect'] = apply_filters( 'pre_term_name', $data['effect'] );
            }
            
            //Finish settings
            if ( ! empty( $data['button_share'] )) {
                $setting['button_share'] = apply_filters( 'single_post_title', $data['button_share'] );
            }
            if ( ! empty( $data['button_invite'] )) {
                $setting['button_invite'] = apply_filters( 'single_post_title', $data['button_invite'] );
            }
            if ( ! empty( $data['button_topscore'] )) {
                $setting['button_topscore'] = apply_filters( 'single_post_title', $data['button_topscore'] );
            }
            if ( ! empty( $data['show_other'] )) {
                $setting['show_other'] = apply_filters( 'single_post_title', $data['show_other'] );
            }
            if ( ! empty( $data['redirect_on'] )) {
                $setting['redirect_on'] = apply_filters( 'single_post_title', $data['redirect_on'] );
            }
            if ( ! empty( $data['redirect_url'] )) {
                $setting['redirect_url'] = apply_filters( 'pre_link_url', $data['redirect_url'] );
            }
            if ( ! empty( $data['redirect_timeout'] )) {
                $setting['redirect_timeout'] = absint( $data['redirect_timeout'] );
            }
            return $setting;
        }
        
        /**
         * Save setting data
         *
         * @param  int $promoId
         * @return void
         */
        public function savePromoSettings( $promoId )
        {
            $promoId = absint( $promoId );
            $data    = $this->_getPromoSettingData();
            $setting = new IPQuiz_Setting( $promoId, '__ipquiz-item-promo-setting' );
            $result  = $setting->setSettings( $data );
            return $result;
        }
        
        /**
         * Get setting data
         *
         * @return array
         */
        protected function _getPromoSettingData()
        {
            $setting = array();
            $data    = array_merge( $_POST, $_GET );
            
            //Content
            if ( ! empty( $data['quiz_url'] )) {
                $setting['quiz_url'] = apply_filters( 'pre_link_url', $data['quiz_url'] );
            }
            if ( ! empty( $data['promo_title'] )) {
                $setting['title'] = apply_filters( 'single_post_title', $data['promo_title'] );
            }
            if ( ! empty( $data['content-desc'] )) {
                $setting['description'] = $data['content-desc'];
            }
            if ( ! empty( $data['image'] )) {
                $setting['image'] = absint( $data['image'] );
            }
            
            //Setting
            if ( isset( $data['size'] )) {
                $setting['size'] = apply_filters( 'pre_term_name', $data['size'] );
            }
            if ( isset( $data['size_height'] )) {
                $setting['size_height'] = absint( $data['size_height'] );
            }
            if ( isset( $data['size_width'] )) {
                $setting['size_width'] = absint( $data['size_width'] );
            }
            if ( isset( $data['design'] )) {
                $setting['design'] = apply_filters( 'pre_term_name', $data['design'] );
            }
            if ( isset( $data['bg_color'] )) {
                $setting['bg_color'] = preg_replace( '@[^a-zA-Z0-9\#]@', '', $data['bg_color'] );
            }
            if ( isset( $data['font_color'] )) {
                $setting['font_color'] = preg_replace( '@[^a-zA-Z0-9\#]@', '', $data['font_color'] );
            }
            if ( ! empty( $data['font_size'] )) {
                $setting['font_size'] = apply_filters( 'pre_term_name', $data['font_size'] );
            }
            if ( ! empty( $data['border'] )) {
                $setting['border'] = apply_filters( 'single_post_title', $data['border'] );
            }
            if ( ! empty( $data['border_size'] )) {
                $setting['border_size'] = absint( $data['border_size'] );
            }
            if ( isset( $data['border_color'] )) {
                $setting['border_color'] = preg_replace( '@[^a-zA-Z0-9\#]@', '', $data['border_color'] );
            }
            if ( ! empty( $data['close_link'] )) {
                $setting['close_link'] = apply_filters( 'single_post_title', $data['close_link'] );
            }
            if ( isset( $data['close_text'] )) {
                $setting['close_text'] = apply_filters( 'single_post_title', $data['close_text'] );
            }
            return $setting;
        }
    }
}