<?php
/**
 * Class for handle ajax request
 */
if ( ! class_exists( 'IPQuiz_Ajax' )) {

    class IPQuiz_Ajax
    {
        /**
         * Take ajax action name
         *
         * @var string
         */
        protected $_action;
        
        /**
         * Take ajax action handlers
         *
         * @var array
         */
        protected $_handlers = array(
            'ipquiz_upload'          => 'doAjaxUpload',
            'ipquiz_clearimage'      => 'doAjaxClearImage',
            'ipquiz_save'            => 'save',
            'ipquiz_delete'          => 'delete',
            'ipquiz_savepromo'       => 'savePromo',
            'ipquiz_deletepromo'     => 'deletePromo',
            'ipquiz_addpostpromo'    => 'addPostPromo',
            'ipquiz_editpostpromo'   => 'editPostPromo',
            'ipquiz_getpostpromo'    => 'getPostPromo',
            'ipquiz_deletepostpromo' => 'deletePostPromo',
            'ipquiz_savesetting'     => 'saveSetting',
            'ipquiz_getchart'        => 'getChart',
            
            'ipquiz_fblogin'         => 'fbLogin',
            'ipquiz_fbget'           => 'fbGet',
            'ipquiz_choice'          => 'choice',
            'ipquiz_frontfinish'     => 'frontFinish',
            'ipquiz_topscore'        => 'getTopscore'
        );
        
        /**
         * Class constructor
         *
         * @return void
         */
        public function __construct()
        {
        }
        
        /**
         * Get action name
         *
         * @return string
         */
        public function getAction()
        {
            return $this->_action;
        }
        
        /**
         * Set action
         *
         * @param  string $action
         * @return IPShortcode_Ajax
         */
        public function setAction( $action )
        {
            $this->_action = sanitize_title( $action );
        }
        
        /**
         * Execute request
         *
         * @param  string $action
         * @return IPShortcode_Ajax
         */
        public function exec()
        {
            $data = array_merge( $_GET, $_POST );
            if ( empty( $data['doajax'] ) || empty( $data['ipquiz'] ) || empty( $data['action'] ) ) {
                return true;
            }
            $this->setAction( $data['action'] );
            
            $action   = $this->getAction();
            $handlers = (array)$this->_handlers;
            if ( array_key_exists( $action, $handlers )) {
                $handler = $handlers[$action];
                if ( method_exists( $this, $handler )) {
                    $this->$handler();
                }
            }
            return true;
        }
        
        /**
         * FB Login
         *
         * @return void
         */
        public function fbLogin( $infb = false )
        {
            $model   = new IPQuiz_Model();
            $status  = 0;
            $parcId  = 0;
            $content = '';
            if ( ! empty( $_POST['fbid'] ) && ! empty( $_POST['quiz'] ) && ! empty( $_POST['fbdata'] )) 
            {
                $fbid   = absint( $_POST['fbid'] );
                $quizId = absint( $_POST['quiz'] );
                $fbdata = json_decode( stripslashes( $_POST['fbdata'] ), true );
                if ( ! $user = $model->getFBUser( $fbid ) ) 
                {
                    $toInsert = array(
                        'facebook_id'   => $fbid,
                        'name'          => $fbdata['name'],
                        'email'         => $fbdata['email'],
                        'link'          => $fbdata['link'],
                        'join_date'     => date( 'c' ),
                        'meta'          => maybe_serialize( $fbdata )
                    );
                    $result = $model->insertParticipant( $toInsert );
                    if ( $result ) {
                        $parcId = $result;
                    }
                } else {
                    $parcId = $user->parc_id;
                }
                if ( $quiz = get_post( $quizId )) 
                {
                    $factory = new IPQuiz_Factory();
                    $content = $factory->getUserQuizProgress( $quiz, $parcId, $infb );
                }
                $status = 1;
                
                //Get step
                $clast   = false;
                $step    = 0;
                $model   = new IPQuiz_Model();
                $choices = $model->getChoices( $quizId, $parcId );
                if ( $choices ) {
                    $ctemp = $choices;
                    $clast = array_pop( $ctemp );
                    $step  = count( $choices );
                }
                if ( ! $clast || $clast->cho_status != '2' ) {
                    $step++;
                }
                
                //Save data progress
                if ( ! $model->getProgress( $quizId, $parcId ) ) 
                {
                    $toInsert = array(
                        'quiz_id'    => $quizId,
                        'parc_id'    => $parcId,
                        'pro_update' => date('c'),
                        'pro_point'  => 0,
                        'pro_step'   => 1,
                        'pro_status' => 1
                    );
                    $model->insertProgress( $toInsert );
                }
            }
            echo json_encode( array(
                'status'  => $status,
                'parc'    => $parcId,
                'step'    => $step,
                'content' => $content
            ) );
            die();
        }
        
        /**
         * FB Get
         *
         * @return void
         */
        public function fbGet()
        {
            $this->fbLogin( true );
        }
        
        /**
         * Get top score data
         *
         * @return void
         */
        public function getTopscore()
        {
            $status   = 0;
            $content  = '<div class="ipquiz-topscore">' .
                            '<div class="ipquiz-topscore-inner">' .
                                '<h2><i class="fa fa-trophy"></i> ' . __( 'Top Scores', 'ipquiz' ) . '</h2>' .
                                '<div class="ipquiz-tops-list">';
            
            if ( ! empty( $_POST['quizid'] ) && ! empty( $_POST['parcid'] ))
            {
                $status = 1;
                $model  = new IPQuiz_Model();
                $quizId = absint( $_POST['quizid'] );
                $parcId = absint( $_POST['parcid'] );
                $quiz   = get_post( $quizId );
                $parc   = $model->getParticipant( $parcId );
                if ( $quiz && $parc ) 
                {
                    $topScores = $model->getQuizParticipants( $quizId );
                    if ( ! empty( $topScores )) 
                    {
                        $order    = 1;
                        $content .= '<ol>';
                        foreach( $topScores as $data )
                        {
                            $class = 'ipquiz-tops-item';
                            if ( $data->parc_id == $parcId ) {
                                $class .= ' ipquiz-tops-current';
                            }
                            $photosrc = 'http://graph.facebook.com/' . $data->facebook_id . '/picture?type=square';
                            $content .= '<li class="' . $class . '">' .
                                            '<div class="ipquiz-tops-inner ipquiz-clear">' .
                                                '<div class="ipquiz-number">' . $order . '</div>' .
                                                '<div class="ipquiz-info ipquiz-clear">' . 
                                                    '<img src="' . $photosrc . '" alt="' . $data->name . '">' .
                                                    '<a href="' . $data->link . '">' . $data->name . '</a>' .
                                                '</div>' .
                                                '<div class="ipquiz-point">' . $data->pro_point . '</div>' .
                                            '</div>' .
                                        '</li>';
                            $order++;
                        }
                        $content .= '</ol>';
                    }
                }
            } else
            {
                $content .= '<p class="ipquiz-tops-empty"><em>' . __( 'No one has followed this quiz', 'ipquiz' ) . '</em></p>';
            }
            $content .= '</div>';
            $content .= '</div>';
            
            echo json_encode( array(
                'status'  => $status,
                'content' => $content
            ) );
            die();
        }
        
        /**
         * Save setting
         *
         * @return void
         */
        public function saveSetting()
        {
            $status = 1;
            if ( ! empty( $_POST['fb_appid'] )) {
                update_option( 'ipquiz_fb_appid', $_POST['fb_appid'] );
            }
            if ( isset( $_POST['share_message'] )) {
                update_option( 'ipquiz_share_message', $_POST['share_message'] );
            }
            if ( isset( $_POST['invite_message'] )) {
                update_option( 'ipquiz_invite_message', $_POST['invite_message'] );
            }
            echo json_encode( array(
                'status'    => $status,
                'message'   => __( 'Successfully save settings' )
            ) );
            die();
        }
        
        /**
         * Choice handler
         *
         * @return void
         */
        public function choice()
        {
            $status = 0;
            if ( $this->_saveChoice() ) {
                $status = 1;
            }
            echo json_encode( array(
                'status'  => $status
            ) );
            die();
        }
        
        /**
         * Save choice
         *
         * @param  boolean $finish
         * @return void
         */
        protected function _saveChoice( $finish = false )
        {
            $result = false;
            if ( ! empty( $_POST['quiz'] ) && ! empty( $_POST['question'] ) && ! empty( $_POST['parc'] )) 
            {
                $model   = new IPQuiz_Model();
                $quizId  = absint( $_POST['quiz'] );
                $qtnId   = absint( $_POST['question'] );
                $parcId  = absint( $_POST['parc'] );
                $quiz    = get_post( $quizId );
                $quest   = $model->getQuestion( $qtnId );
                $parc    = $model->getParticipant( $parcId );
                if ( $quiz && $quest && $parc ) 
                {
                    $setting = new IPQuiz_Setting( $quiz->ID, '__ipquiz-item-setting' );
                    $selects = array();
                    foreach( $_POST as $name => $value ) {
                        if ( strpos( $name, 'answer_value_' ) !== false ) {
                            if ( $value != 0 && $value != '0' ) {
                                $ansId = str_replace( 'answer_value_', '', $name );
                                $selects[] = $ansId;
                            }
                        }
                    }
                    
                    if ( ! isset( $_POST['timeout'] )) 
                    {
                        $correct = 0;
                        $wrong   = 0;
                        if ( $quest->qtn_type == 'all' ) {
                            $correct++;
                        } elseif ( $quest->qtn_type == 'none' ) {
                            $wrong++;
                        } else
                        {
                            $answers = array();
                            if ( $ansData = $model->getAnswers( $qtnId ) ) {
                                foreach( $ansData as $ans ) {
                                    if ( $ans->ans_status == 1 ) {
                                        $answers[] = $ans->ans_id;
                                    }
                                }
                            }
                            foreach( $selects as $sel ) {
                                if ( in_array( $sel, $answers )) {
                                    $correct++;
                                } else {
                                    $wrong++;
                                }
                            }
                        }
                        if ( $correct > 0 && $wrong < 1 ) {
                            $status = 1;
                        } else {
                            $status = 0;
                        }
                        
                    } else {
                        $status = 2;
                    }
                    
                    //Save choice data
                    $toInsert = array(
                        'parc_id'       => $parcId,
                        'quiz_id'       => $quizId,
                        'qtn_id'        => $qtnId,
                        'cho_date'      => date( 'c' ),
                        'cho_time'      => time(),
                        'cho_ip'        => $_SERVER['REMOTE_ADDR'],
                        'cho_browser'   => $_SERVER['HTTP_USER_AGENT'],
                        'cho_status'    => $status
                    );
                    $choId = $model->insertChoice( $toInsert );
                    if ( $choId && ! empty( $selects ) ) 
                    {
                        foreach( $selects as $id ) 
                        {
                            $toInsert = array(
                                'cho_id'    => $choId,
                                'qtn_id'    => $qtnId,
                                'ans_id'    => $id
                            );
                            $model->insertChoiceAnswer( $toInsert );
                        }
                    }
                    
                    //Save progress data
                    $step = 1;
                    if ( ! empty( $_POST['step'] )) {
                        $step = absint( $_POST['step'] );
                    }
                    if ( $progress = $model->getProgress( $quizId, $parcId ) )
                    {
                        $point = $progress->pro_point;
                        $plus  = $setting->getSetting( 'correct_point', 1 );
                        $minus = $setting->getSetting( 'wrong_point', 0 );
                        if ( $status == 1 ) {
                            $point += $plus;
                        } else {
                            $point -= $minus;
                        }
                        
                        $toUpdate = array(
                            'pro_update' => date( 'c' ),
                            'pro_point'  => $point,
                            'pro_step'   => $step
                        );
                        if ( $finish ){
                            $toUpdate['pro_status'] = 2;
                        }
                        $model->updateProgress( $quizId, $parcId, $toUpdate );
                    }
                    $result = true;
                }
            }
            return $result;
        }
        
        /**
         * Front finish handler
         *
         * @return void
         */
        public function frontFinish()
        {
            //Save choice
            $model  = new IPQuiz_Model();
            $status = 1;
            $result = '';
            if ( $this->_saveChoice( true ) ) 
            {
                $quizId    = absint( $_POST['quiz'] );
                $quiz      = get_post( $quizId );
                $parcId    = absint( $_POST['parc'] );
                $parc      = $model->getParticipant( $parcId );
                $setting   = new IPQuiz_Setting( $quiz->ID, '__ipquiz-item-setting' );
                $questions = $model->getQuestions( $quiz->ID );
            
                $factory = new IPQuiz_Factory();
                $result  = $factory->getFinish( $quiz, $parc, $questions, $setting );
                
                //Save user point
                if ( $progress = $model->getProgress( $quizId, $parcId ) )
                {
                    $upoint  = $parc->point;
                    $upoint += $progress->pro_point;
                    $toUpdate = array(
                        'point' => $upoint
                    );
                    $model->updateParticipant( $parcId, $toUpdate );
                }
            }
            echo json_encode( array(
                'status'  => $status,
                'html'    => $result
            ) );
            die();
        }
        
        /**
         * Add/Save quiz
         *
         * @return void
         */
        public function save()
        {
            $isAdd = false;
            $error = true;
            if ( ! empty( $_GET['add'] )) {
                $isAdd = true;
            }
            if ( ! empty( $_POST['quizid'] )) {
                $quizId = absint( $_POST['quizid'] );
                if ( $quiz = get_post( $quizId ) ) {
                    $this->_updateQuiz( $quiz );
                } else {
                    $quizId = 0;
                }
            } else {
                $quizId = $this->_addQuiz();
            }
            if ( $quizId )
            {
                $error = false;
                $model = new IPQuiz_Model();
                //Save Question
                $model->saveQuestion( $quizId );
                //Save settings
                $model->saveQuizSettings( $quizId );
            }
            if ( $error ) {
                $status  = 0;
                $quizId  = 0;
                if ( $isAdd ) {
                    $message = __( 'Failed to create a quiz.', 'ipquiz' );
                } else {
                    $message = __( 'Failed to save quiz.', 'ipquiz' );
                }
            } else {
                $status = 1;
                if ( $isAdd ) {
                    $message = __( 'Successfully create a quiz.', 'ipquiz' );
                } else {
                    $message = __( 'Successfully to save quiz.', 'ipquiz' );
                }
            }
            echo json_encode( array(
                'status'  => $status,
                'message' => $message,
                'quizid'  => $quizId
            ) );
            die();
        }
        
        /**
         * Add new Quiz
         *
         * @return int
         */
        protected function _addQuiz()
        {
            global $user_ID;
            
            if ( empty( $_POST['title'] )) {
                $title = 'Unknown';
            } else {
                $title = apply_filters( 'single_post_title', $_POST['title'] );
            }
            $status = 'publish';
            if ( empty( $_POST['status'] )) {
                $status = apply_filters( 'pre_term_name', $_POST['status'] );
            }
            $dataPost = array(
                'post_status' => $status, 
                'post_type'   => 'ipquiz-item', 
                'post_author' => $user_ID,
                'post_title'  => $title
            );
            if ( ! empty( $_POST['content-desc'] )) {
                $dataPost['post_content'] = $_POST['content-desc'];
            }
            $result = wp_insert_post( $dataPost );
            return $result;
        }
        
        /**
         * Update Quiz
         *
         * @param  object $quiz
         * @return boolean
         */
        protected function _updateQuiz( $quiz )
        {
            $result   = 0;
            $dataPost = array();
            if ( ! empty( $_POST['title'] ) && $_POST['title'] != $quiz->post_title ) {
                $dataPost['post_title'] = apply_filters( 'single_post_title', $_POST['title'] );
            }
            if ( ! empty( $_POST['status'] ) && $_POST['status'] != $quiz->post_status ) {
                $dataPost['post_status'] = apply_filters( 'pre_term_name', $_POST['status'] );
            }
            if ( ! empty( $_POST['content-desc'] ) && $_POST['content-desc'] != $quiz->post_content ) {
                $dataPost['post_content'] = $_POST['content-desc'];
            }
            if ( ! empty( $dataPost )) {
                $dataPost['ID'] = $quiz->ID;
                $result = wp_update_post( $dataPost );
            }
            return $result;
        }
        
        /**
         * Add/Save quiz promo
         *
         * @return void
         */
        public function savePromo()
        {
            $isAdd = false;
            $error = true;
            if ( ! empty( $_GET['add'] )) {
                $isAdd = true;
            }
            if ( ! empty( $_POST['promoid'] )) {
                $promoId = absint( $_POST['promoid'] );
                if ( $promo = get_post( $promoId ) ) {
                    $this->_updateQuizPromo( $promo );
                } else {
                    $promoId = 0;
                }
            } else {
                $promoId = $this->_addQuizPromo();
            }
            if ( $promoId )
            {
                $error = false;
                $model = new IPQuiz_Model();
                $model->savePromoSettings( $promoId );
            }
            if ( $error ) {
                $status   = 0;
                $promoId  = 0;
                if ( $isAdd ) {
                    $message = __( 'Failed to create a quiz promo.', 'ipquiz' );
                } else {
                    $message = __( 'Failed to save quiz promo.', 'ipquiz' );
                }
            } else {
                $status = 1;
                if ( $isAdd ) {
                    $message = __( 'Successfully create a quiz promo.', 'ipquiz' );
                } else {
                    $message = __( 'Successfully to save quiz promo.', 'ipquiz' );
                }
            }
            echo json_encode( array(
                'status'   => $status,
                'message'  => $message,
                'promoid'  => $promoId
            ) );
            die();
        }
        
        /**
         * Add new Quiz Promo
         *
         * @return int
         */
        protected function _addQuizPromo()
        {
            global $user_ID;
            
            if ( empty( $_POST['title'] )) {
                $title = 'Unknown';
            } else {
                $title = apply_filters( 'single_post_title', $_POST['title'] );
            }
            $status = 'publish';
            if ( empty( $_POST['status'] )) {
                $status = apply_filters( 'pre_term_name', $_POST['status'] );
            }
            $dataPost = array(
                'post_status' => $status, 
                'post_type'   => 'ipquiz-promo-item', 
                'post_author' => $user_ID,
                'post_title'  => $title
            );
            $result = wp_insert_post( $dataPost );
            return $result;
        }
        
        /**
         * Update Quiz promo
         *
         * @param  object $promo
         * @return boolean
         */
        protected function _updateQuizPromo( $promo )
        {
            $result   = 0;
            $dataPost = array();
            if ( ! empty( $_POST['title'] ) && $_POST['title'] != $promo->post_title ) {
                $dataPost['post_title'] = apply_filters( 'single_post_title', $_POST['title'] );
            }
            if ( ! empty( $_POST['status'] ) && $_POST['status'] != $promo->post_status ) {
                $dataPost['post_status'] = apply_filters( 'pre_term_name', $_POST['status'] );
            }
            if ( ! empty( $dataPost )) {
                $dataPost['ID'] = $promo->ID;
                $result = wp_update_post( $dataPost );
            }
            return $result;
        }
        
        /**
         * Processing ajax for delete quiz
         *
         * @return void
         */
        public function delete()
        {
            $item = false;
            if ( ! empty( $_REQUEST['id'] )) {
                $id   = absint( $_REQUEST['id'] );
                $item = get_post( $id );
            }
            if ( ! $item ) {
                echo json_encode( array(
                    'status'   => 0,
                    'message'  => __( 'Error system', 'ipquiz' )
                ) );
                die();
            }
            $model = new IPQuiz_Model();
            if ( $questions = $model->getQuestions( $item->ID )) 
            {
                foreach( $questions as $qtn )
                {
                    $model->deleteQuestion( $qtn->qtn_id );
                    $model->deleteAnswers( $qtn->qtn_id );
                }
            }
            
            $result = wp_delete_post( $item->ID, true );
            if ( $result ) {
                echo json_encode( array(
                    'status'  => 1,
                    'message' => __( 'Successfully remove the quiz promo', 'ipquiz' )
                ) );
            } else {
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'Failed to remove the quiz promo', 'ipquiz' )
                ) );
            }
            die();
        }
        
        /**
         * Processing ajax for delete quiz promo
         *
         * @return void
         */
        public function deletePromo()
        {
            $item = false;
            if ( ! empty( $_REQUEST['id'] )) {
                $id   = absint( $_REQUEST['id'] );
                $item = get_post( $id );
            }
            if ( ! $item ) {
                echo json_encode( array(
                    'status'   => 0,
                    'message'  => __( 'Error system', 'ipquiz' )
                ) );
                die();
            }
            $result = wp_delete_post( $item->ID, true );
            if ( $result ) {
                echo json_encode( array(
                    'status'  => 1,
                    'message' => __( 'Successfully remove the quiz promo', 'ipquiz' )
                ) );
            } else {
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'Failed to remove the quiz promo', 'ipquiz' )
                ) );
            }
            die();
        }
        
        /**
         * Processing ajax for add post promo
         *
         * @return void
         */
        public function addPostPromo()
        {
            if ( empty( $_POST['ipquiz_promo'] )) {
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'Failed to add the quiz promo.', 'ipquiz' )
                ) );
                die();
            }
            $promoid = absint( $_POST['ipquiz_promo'] );
            if ( ! $promo = get_post( $promoid )) {
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'The quiz promo is not available.', 'ipquiz' )
                ) );
                die();
            }
            
            if ( $_POST['type'] == 'post' ) {
                $data = $this->_getPostPromo( $promo );
            } else {
                $data = $this->_getHomePromo( $promo );
            }
            $key = 0;
            foreach( $data as $id => $val ) {
                if ( $key < $id ) {
                    $key = $id;
                }
            }
            $key++;
            
            $data[$key] = array(
                'id'        => $promoid,
                'position'  => $_POST['ipquiz_position']
            );
            $toupdate = maybe_serialize( $data );
            if ( $_POST['type'] == 'post' ) {
                update_post_meta( $_POST['postid'], '__ipquiz_post_promos', $toupdate );
            } else {
                update_option( '__ipquiz_home_promos', $toupdate );
            }
            $html = '<li id="postPromo_' . $key . '" class="ipadm-clear ipquiz-promo">'
                        . '<div class="ipquiz-promo-title">'
                            . '<strong>' . $promo->post_title . '</strong>'
                            . '<a href="#" class="ipquiz-promo-edit">'
                                . __( 'Edit', 'ipquiz' )
                            . '</a>'
                        . '</div>'
                        . '<div class="ipquiz-promo-delete">'
                            . '<a href="#postPromo_' . $key . '"><i class="fa fa-times-circle"></i></a>'
                        . '</div>'
                    . '</li>';
            
            echo json_encode( array(
                'id'      => $key,
                'status'  => 1,
                'message' => __( 'Successfully adding the quiz promo.', 'ipquiz' ),
                'html'    => $html
            ) );
            die();
        }
        
        /**
         * Get post promo
         *
         * @return void
         */
        protected function _getPostPromo()
        {
            if ( empty( $_POST['postid'] )) {
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'Failed to get the quiz promo.', 'ipquiz' )
                ) );
                die();
            }
            $postid = absint( $_POST['postid'] );
            if ( ! $post = get_post( $postid )) {
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'The post is not available.', 'ipquiz' )
                ) );
                die();
            }
            $data = array();
            $meta = get_post_meta( $post->ID, '__ipquiz_post_promos', true );
            if ( ! empty( $meta )) {
                $data  = maybe_unserialize( $meta );
                $data  = (array)$data;
            }
            return $data;
        }
        
        /**
         * Get home promo
         *
         * @return void
         */
        protected function _getHomePromo()
        {
            $data  = array();
            $metas = (array)get_option( '__ipquiz_home_promos' );
            if ( ! empty( $metas )) {
                $meta = array_shift( $metas );
                $ids  = maybe_unserialize( $meta );
                $data = (array)$ids;
            }
            return $data;
        }
        
        /**
         * Processing ajax for edit post promo
         *
         * @return void
         */
        public function editPostPromo()
        {
            if ( empty( $_POST['promoid'] )) {
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'Failed to edit the quiz promo.', 'ipquiz' )
                ) );
                die();
            }
            $listid = absint( $_POST['promoid'] );
            $list   = false;
            if ( $_POST['type'] == 'post' )
            {
                $postid = absint( $_POST['postid'] );
                if ( ! $post = get_post( $postid )) {
                    echo json_encode( array(
                        'status'  => 0,
                        'message' => __( 'The post is not available.', 'ipquiz' )
                    ) );
                    die();
                }
                $meta = get_post_meta( $post->ID, '__ipquiz_post_promos', true );
                $data = array();
                if ( ! empty( $meta )) {
                    $data  = maybe_unserialize( $meta );
                    if ( isset( $data[$listid] )) {
                        $list = $data[$listid];
                    }
                }
            } else
            {
                $metas = (array)get_option( '__ipquiz_home_promos' );
                if ( ! empty( $metas )) {
                    $meta = array_shift( $metas );
                    $data = maybe_unserialize( $meta );
                    if ( isset( $data[$listid] )) {
                        $list = $data[$listid];
                    }
                }
            }
            $title = '';
            if ( $list )
            {
                $data[$listid] = array(
                    'id'        => $_POST['ipquiz_promo'],
                    'position'  => $_POST['ipquiz_position']
                );
                $toupdate = maybe_serialize( $data );
                if ( $_POST['type'] == 'post' ) {
                    update_post_meta( $post->ID, '__ipquiz_post_promos', $toupdate );
                } else {
                    update_option( '__ipquiz_home_promos', $toupdate );
                }
                
                $status  = 1;
                if ( $promo = get_post( $_POST['ipquiz_promo'] )) {
                    $title = $promo->post_title;
                }
                $message = __( 'Successfully editing the quiz promo.', 'ipquiz' );
            }
            else {
                $status  = 0;
                $message = __( 'Failed editing the quiz promo.', 'ipquiz' );
            }
            echo json_encode( array(
                'status'  => $status,
                'message' => $message,
                'title'   => $title
            ) );
            die();
        }
        
        /**
         * Get post promo
         *
         * @return void
         */
        public function getPostPromo()
        {
            if ( empty( $_POST['id'] )) {
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'Failed to get quiz promo data.', 'ipquiz' )
                ) );
                die();
            }
            $isGet  = false;
            $listid = absint( $_POST['id'] );
            
            if ( $_POST['type'] == 'post' )
            {
                $postid = absint( $_POST['postid'] );
                if ( ! $post = get_post( $postid )) {
                    echo json_encode( array(
                        'status'  => 0,
                        'message' => __( 'The post is not available.', 'ipquiz' )
                    ) );
                    die();
                }
                $meta  = get_post_meta( $post->ID, '__ipquiz_post_promos', true );
                if ( ! empty( $meta )) 
                {
                    $data = maybe_unserialize( $meta );
                    if ( isset( $data[$listid] )) {
                        $isGet = $data[$listid];
                    }
                }
            } else 
            {
                $metas = (array)get_option( '__ipquiz_home_promos' );
                if ( ! empty( $metas )) {
                    $meta = array_shift( $metas );
                    $data = maybe_unserialize( $meta );
                    if ( isset( $data[$listid] )) {
                        $isGet = $data[$listid];
                    }
                }
            }
            if ( $isGet ) {
                echo json_encode( array(
                    'status'  => 1,
                    'data'    => $isGet,
                ) );
            } else {
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'Failed to get quiz promo data.', 'ipquiz' ),
                ) );
            }
            die();
        }
        
        /**
         * Processing ajax for delete post quiz promo
         *
         * @return void
         */
        public function deletePostPromo()
        {
            if ( empty( $_POST['listid'] )) {
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'Failed to delete the quiz promo.', 'ipquiz' )
                ) );
                die();
            }
            $listid = absint( $_POST['listid'] );
            $data   = array();
            if ( $_POST['type'] == 'post' ) 
            {
                $postid = absint( $_POST['postid'] );
                if ( ! $post = get_post( $postid )) {
                    echo json_encode( array(
                        'status'  => 0,
                        'message' => __( 'The post is not available.', 'ipquiz' )
                    ) );
                    die();
                }
                $meta = get_post_meta( $post->ID, '__ipquiz_post_promos', true );
                if ( ! empty( $meta )) {
                    $data = maybe_unserialize( $meta );
                }
            } else
            {
                $metas = (array)get_option( '__ipquiz_home_promos' );
                if ( ! empty( $metas )) {
                    $meta = array_shift( $metas );
                    $data = maybe_unserialize( $meta );
                }
            }
            if ( ! empty( $data )) {
                if ( isset( $data[$listid] )) 
                {
                    unset( $data[$listid] );
                    $toupdate = maybe_serialize( $data );
                    if ( $_POST['type'] == 'post' ) {
                        update_post_meta( $post->ID, '__ipquiz_post_promos', $toupdate );
                    } else {
                        update_option( '__ipquiz_home_promos', $toupdate );
                    }
                    echo json_encode( array(
                        'status'  => 1,
                        'message' => __( 'Successfully deleting the quiz promo.', 'ipquiz' )
                    ) );
                    die();
                }
            }
            echo json_encode( array(
                'status'  => 0,
                'message' => __( 'Failed deleting the quiz promo.', 'ipquiz' )
            ) );
            die();
        }
        
        /**
         * Processing ajax for upload image
         *
         * @return void
         */
        public function doAjaxUpload()
        {
            if ( ! empty( $_FILES['async-upload'] )) {
            
                $filetype = wp_check_filetype_and_ext( $_FILES['async-upload']['tmp_name'], $_FILES['async-upload']['name'], false );
                if ( ! wp_match_mime_types( 'image', $filetype['type'] ) ) {
                    echo json_encode( array(
                        'status'  => 0,
                        'data'    => array(
                            'message'  => __( 'The uploaded file is not a valid image. Please try again.' ),
                            'filename' => $_FILES['async-upload']['name'],
                        )
                    ) );
                    die();
                }
                $attachment_id = media_handle_upload( 'async-upload', 0, array() );
                if ( is_wp_error( $attachment_id ) ) {
                    echo json_encode( array(
                        'status' => 0,
                        'data'    => array(
                            'message'  => $attachment_id->get_error_message(),
                            'filename' => $_FILES['async-upload']['name'],
                        )
                    ) );
                    die();
                }
                if ( ! $attachment = wp_prepare_attachment_for_js( $attachment_id ) ) {
                    die();
                }
                
                echo json_encode( array(
                    'status'   => 1,
                    'data'     => $attachment
                ));
            }
            die();
        }
        
        /**
         * Processing ajax for clearing image
         *
         * @return void
         */
        public function doAjaxClearImage()
        {
            $attach = false;
            if ( ! empty( $_REQUEST['attach'] )) {
                $attachid = absint( $_REQUEST['attach'] );
                $attach   = get_post( $attachid );
            }
            if ( ! $attach ) {
                echo json_encode( array(
                    'status'  => 0,
                    'data'    => array(
                        'message'  => __( 'Error system' )
                    )
                ) );
                die();
            }
            
            $delete = wp_delete_attachment( $attachid, true );
            if ( $delete ) {
                echo json_encode( array(
                    'status' => 1,
                    'data'   => array(
                        'message'  => __( 'Successfully clearing image' )
                    )
                ));
            
            } else {
                echo json_encode( array(
                    'status'  => 0,
                    'data'    => array(
                        'message'  => __( 'Error clearing image' )
                    )
                ) );
            }
            die();
        }
        
        /**
         * Get chart
         *
         * @return void
         */
        public function getChart()
        {
            $data = array_merge( $_POST, $_GET );
            if ( ! empty( $data['start'] ) && ! empty( $data['end'] )) 
            {
                $timeEnd   = strtotime( $data['end'] );
                $timeStart = strtotime( $data['start'] );
                $dateStart = gmdate("Y-m-d", $timeStart);
                $dateEnd   = gmdate("Y-m-d", $timeEnd);
                $model     = new IPQuiz_Model();
                $chart     = array();
                $results   = $model->getStatisticData( $dateStart, $dateEnd );
                
                if ( ! empty( $results )) {
                    foreach( $results as $data ) {
                        $timestamp = strtotime( $data->gdate );
                        $phpdate   = getdate($timestamp);
                        $chart[]   = array(
                            'total_parc'    => (int)$data->total_parc,
                            'year'          => $phpdate['year'],
                            'mon'           => $phpdate['mon'] - 1,
                            'day'           => $phpdate['mday'],
                        );
                    }
                }
                echo json_encode( array(
                    'status'            => 1,
                    'labelDate'         => __( 'Date', 'ipadmak' ),
                    'labelParc'         => __( 'Participations', 'ipadmak' ),
                    'data'              => $chart
                ) );
            } else {
            
                echo json_encode( array(
                    'status' => 0
                ) );
            }
            die();
        }
    }
}