<?php
if ( ! class_exists( 'IPQuiz_FactoryPromo' )) {

    /**
     * Class for create quiz promo
     */
    class IPQuiz_FactoryPromo
    {
        /**
         * Create widget content
         *
         * @param  array  $atts
         * @param  string $instance
         * @return string
         */
        public function widget( $atts, $instance )
        {
            $result = '';
            if ( ! empty( $instance['promo'] )) {
                $promoId = absint( $instance['promo'] );
                if ( $promo = get_post( $promoId )) {
                    $result = '<aside class="widget ipquiz-widget">' . 
                                    $this->getPromoItem( $promo, $instance ) .
                               '</aside>';
                }
            }
            return $result;
        }
        
        /**
         * Create ad content
         *
         * @return void
         */
        public function render()
        {
            global $post;
            
            $posdata = array(
                'topleft'       => array(),
                'topcenter'     => array(),
                'topright'      => array(),
                'bottomleft'    => array(),
                'bottomcenter'  => array(),
                'bottomright'   => array()
            );
            $promos = $this->getPromoPosts();
            if ( ! empty( $promos )) {
                foreach( $promos as $data )
                {
                    $id = $data['id'];
                    if ( $promo = get_post( $id ) ) {
                        if ( $item = $this->getPromoItem( $promo, $data ) )
                        {
                            $pos  = 'topleft';
                            if ( ! empty( $data['position'] )) {
                                $pos = $data['position'];
                            }
                            if ( isset( $posdata[$pos] )) {
                                $posdata[$pos][] = $item;
                            }
                        }
                    }
                }
            }
            $result = '<div id="ipquiz-promo-popup">';
            foreach( $posdata as $section => $items )
            {
                if ( ! empty( $items ) ) 
                {
                    $result .= '<div class="ipquiz-promo-section ipquiz-promo-' . $section . '">';
                    foreach( $items as $item ) {
                        $result .= $item;
                    }
                    $result .= '</div>';
                }
            }
            $result .= '</div>';
            echo $result;
        }
        
        /**
         * Get promos list
         *
         * @return string
         */
        public function getPromoPosts()
        {
            global $post;
            
            $promos = array();
            if ( is_home() ) {
                $metas = (array)get_option( '__ipquiz_home_promos' );
                if ( ! empty( $metas )) {
                    $meta   = array_shift( $metas );
                    $ids    = maybe_unserialize( $meta );
                    $promos = (array)$ids;
                }
            } elseif ( ! empty( $post )) {
                $meta = get_post_meta( $post->ID, '__ipquiz_post_promos', true );
                if ( ! empty( $meta )) {
                    $data   = maybe_unserialize( $meta );
                    $promos = (array)$data;
                }
            }
            foreach( $promos as $idx => $promo ) {
                if ( empty( $promo )) {
                    unset( $promos[$idx] );
                }
            }
            return $promos;
        }
        
        /**
         * Get promo content
         *
         * @param object $promo
         * @param array  $attr
         * @return string
         */
        public function getPromoItem( $promo = false, $attr = array() )
        {
            $promoId = 0;
            if ( $promo ) {
                $promoId = $promo->ID;
            }
            $setting = new IPQuiz_Setting( $promoId, '__ipquiz-item-promo-setting' );
            if ( $promo && $promo->post_status != 'publish' ) {
                return false;
            }
            $setting->addSettings( $attr );
            
            $class = $this->_getPromoClass( $setting );
            $style = $this->_getPromoStyle( $setting );
            
            $result = '<div data-id="' . $promoId . '" class="' . $class . '"' . $style . '>' .
                            '<div class="ipquiz-inner">' . 
                                $this->_getPromoContent( $promo, $setting ) . 
                            '</div>' .
                            $this->_getCloseLink( $promo, $setting ) .
                        '</div>';
            return $result;
        }
        
        /**
         * Get promo class
         *
         * @param object $setting
         * @return string
         */
        protected function _getPromoClass( $setting )
        {
            $class = 'ipquiz-promo';
            if ( $font = $setting->getSetting( 'font_size', 'medium' ) ) {
                $class .= ' ipquiz-font-' . $font;
            }
            $size   = $setting->getSetting( 'size', 'auto' );
            $class .= ' ipquiz-size-' . $size;
            $design = $setting->getSetting( 'design', 'flat' );
            $class .= ' ipquiz-design-' . $design;
            
            if ( $setting->getSetting( 'image' ) ) {
                $class .= ' ipquiz-with-img';
            }
            return $class;
        }
        
        /**
         * Get promo style
         *
         * @param object $setting
         * @return string
         */
        protected function _getPromoStyle( $setting )
        {
            $style  = 'background-color: ' . $setting->getSetting( 'bg_color', '#ffffff' ) . ';';
            $style .= ' color: ' . $setting->getSetting( 'font_color', '#000000' ) . ';';
            
            if ( $setting->getSetting( 'border', 'false' ) == 'true' ) {
                $size   = $setting->getSetting( 'border_size', 1 );
                $color  = $setting->getSetting( 'border_color', '#000000' );
                $style .= ' border: ' . $size . 'px solid ' . $color . ';';
            }
            
            $size = $setting->getSetting( 'size', 'auto' );
            if ( $size == 'custom' ) {
                $style .= ' height: ' . $setting->getSetting( 'size_height', 0 ) . 'px;';
                $style .= ' width: ' . $setting->getSetting( 'size_width', 0 ) . 'px;';
            }
            
            if ( ! empty( $style )) {
                $style = ' style="' . $style . '"';
            }
            return $style;
        }
        
        /**
         * Get promo content
         *
         * @param object $promo
         * @param object $setting
         * @return string
         */
        protected function _getPromoContent( $promo, $setting )
        {
            $link    = $setting->getSetting( 'quiz_url', '#' );
            $content = $this->_getLayout( $promo, $setting );
            $result  = '<div class="ipquiz-content-wrap">' .
                            '<a class="ipquiz-clear" href="' . $link . '">' . $content . '</a>' .
                        '</div>';
            return $result;
        }
        
        /**
         * Get promo layout content
         *
         * @param object $promo
         * @param object $setting
         * @return string
         */
        protected function _getLayout( $promo, $setting )
        {
            $content  = '<div class="ipquiz-thumb">' .
                            $this->_getContentImage( $promo, $setting ) .
                        '</div>' .
                        '<div class="ipquiz-content-main">' .
                            '<div class="ipquiz-content-inner">' .
                                $this->_getContentTitle( $promo, $setting ) .
                                $this->_getContentDesc( $promo, $setting ) .
                            '</div>' .
                        '</div>';
            return $content;
        }
        
        /**
         * Get content image
         *
         * @param object $promo
         * @param object $setting
         * @return string
         */
        protected function _getContentImage( $promo, $setting )
        {
            $content  = '';
            if ( $imgId = $setting->getSetting( 'image' ) ) {
                $imgId = absint( $imgId );
                if ( $image = get_post( $imgId )) {
                    $content .= '<img src="' . $image->guid . '" alt="">';
                }
            }
            return $content;
        }
        
        /**
         * Get content title
         *
         * @param object $promo
         * @param object $setting
         * @return string
         */
        protected function _getContentTitle( $promo, $setting )
        {
            $content  = '';
            if ( $title = $setting->getSetting( 'title' ) ) {
                $content .= '<div class="ipquiz-promo-title">' .
                                '<h3>' . $title . '</h3>' .
                            '</div>';
            }
            return $content;
        }
        
        /**
         * Get content description
         *
         * @param object $promo
         * @param object $setting
         * @return string
         */
        protected function _getContentDesc( $promo, $setting )
        {
            $content  = '';
            if ( $desc = $setting->getSetting( 'description' ) ) {
                $content .= '<div class="ipquiz-promo-desc">' . $desc . '</div>';
            }
            return $content;
        }
        
        /**
         * Get close link
         *
         * @param object $promo
         * @param object $setting
         * @return string
         */
        protected function _getCloseLink( $promo, $setting )
        {
            $close = '';
            if ( $setting->getSetting( 'close_link', 'false' ) == 'true' )
            {
                $text  = $setting->getSetting( 'close_text', 'x close' );
                $close = '<div class="ipquiz-promo-close">'.
                            '<a class="ipquiz-promo-close-link" href="#">' . $text . '</a>' .
                         '</div>';
            }
            return $close;
        }
    }
}