<?php
if ( ! class_exists( 'IPQuiz_Factory' )) {

    /**
     * Class for create popup
     */
    class IPQuiz_Factory
    {
        /**
         * Take model object
         *
         * @var IPQuiz_Model
         */
        public $model;
        
        /**
         * Class constructor
         *
         * @return void
         */
        public function __construct()
        {
            $this->model = new IPQuiz_Model();
        }
        
        /**
         * Create shortcode content
         *
         * @param  array  $atts
         * @param  string $content
         * @return string
         */
        public function shortcode( $atts, $content = '' )
        {
            $result = '';
            if ( ! empty( $atts['quiz'] )) {
                $quizId = absint( $atts['quiz'] );
                if ( $quiz = get_post( $quizId )) {
                    $result = '<div class="ipquiz-shortcode">' . 
                                    $this->getQuizItem( $quiz, $atts ) .
                               '</div>';
                }
            }
            return $result;
        }
        
        /**
         * Get user quiz progress
         *
         * @param  object  $quiz
         * @param  int     $parcId
         * @param  boolean $infb
         * @return string
         */
        public function getUserQuizProgress( $quiz, $parcId, $infb = false )
        {
            $result    = '';
            $user      = $this->model->getParticipant( $parcId );
            $questions = $this->model->getQuestions( $quiz->ID );
            $setting   = new IPQuiz_Setting( $quiz->ID, '__ipquiz-item-setting' );
            if ( $user && $questions ) 
            {
                if ( $infb ) {
                    $result .= $this->_getQuizInfo( $quiz );
                }
                $result .= $this->getQuestions( $quiz, $user, $questions, $setting );
            }
            return $result;
        }
        
        /**
         * Create facebook page content
         *
         * @param  object $quiz
         * @return string
         */
        public function facebook( $quiz )
        {
            $path = IPQUIZ_PATH . 'includes/facebook.php';
            if ( file_exists( $path )) 
            {
                $this->appId = get_option( 'ipquiz_fb_appid' );
                $this->quiz  = $quiz;
                $this->content = $this->getQuizItem( $quiz, array() );
                
                include( $path );
            }
        }
        
        /**
         * Get quiz item
         *
         * @param  object  $quiz
         * @param  array   $atts
         * @param  boolean $intro
         * @return string
         */
        public function getQuizItem( $quiz, $atts = array(), $intro = true )
        {
            //Set settings
            $setting = new IPQuiz_Setting( $quiz->ID, '__ipquiz-item-setting' );
            if ( ! empty( $atts )) {
                $setting->addSettings( $atts );
            }
            $effect  = $setting->getSetting( 'effect', '0' );
            $caudio  = $setting->getSetting( 'correct_audio', '0' );
            $waudio  = $setting->getSetting( 'wrong_audio', '0' );
            
            //Create quiz content
            $class   = $this->_getQuizClass( $quiz, $setting );
            $style   = $this->_getQuizStyle( $quiz, $setting );
            $result  = '';
            if ( $intro ) {
                $result .= $this->_fbScript( $setting );
            }
            $result .= '<div id="ipquiz-item-' . $quiz->ID . '" class="' . $class . '" style="' . $style . '">' .
                            '<div class="ipquiz-inner">' .
                                '<div class="ipquiz-main">' .
                                    '<form method="post" action="">' .
                                        '<div class="ipquiz-content">' .
                                            $this->_getQuizContent( $quiz, $setting, $intro ) . 
                                        '</div>' .
                                        '<div class="ipquiz-settings"><div><div><div><div>' .
                                            '<input type="hidden" class="ipquiz-effect" name="effect" value="' . $effect . '"/>' .
                                            $this->_getAudio( $caudio, 'ipquiz-correct-audio' ) .
                                            $this->_getAudio( $waudio, 'ipquiz-wrong-audio' ) .
                                            '<input type="hidden" class="ipquiz-st-118-2" name="st_118_2" value="0"/>' .
                                            '<input type="hidden" class="ipquiz-current-step" name="step" value="1"/>' .
                                        '</div></div></div></div></div>' .
                                    '</form>' .
                                    '<div class="ipquiz-loader"><span><span></span></span></div>' .
                                '</div>' .
                            '</div>' .
                       '</div>';
            return $result;
        }
        
        /**
         * Get audio
         *
         * @param object  $setting
         * @return string
         */
        protected function _fbScript( $setting )
        {
            global $ipquiz_fbscript;
            
            $appId  = get_option( 'ipquiz_fb_appid' );
            $script = '';
            if ( ! $ipquiz_fbscript ) {

$script .= '<div id="fb-root"></div>' .
'<script>
    window.fbAsyncInit = function() {
        FB.init({ 
            appId: "' . $appId . '",
            status: true, 
            cookie: true,
            xfbml: true,
            oauth: true
        });
        jQuery(".ipquiz-btn-fblogin").each( function() 
        {
            var button = jQuery(this);
            button.click( function() {
                FB.login(function(response) {
                    if (response.authResponse) {
                        ipquiz_FB.dologin( response.authResponse, button );
                    } else {
                        console.log("Login cancelled or did not fully authorize.");
                    }
                }, {scope:"email"});
                return false;
            });
        });
    };
    (function() {
        var e = document.createElement("script"); e.async = true;
        e.src = document.location.protocol 
                + "//connect.facebook.net/en_US/all.js";
        document.getElementById("fb-root").appendChild(e);
    }());
</script>';

                $ipquiz_fbscript = true;
            }
            return $script;
        }
        
        /**
         * Get audio
         *
         * @param int    $id
         * @return string
         */
        protected function _getAudio( $id, $class )
        {
            $result = '';
            if ( $id != 0 ) {
                $base    = IPQUIZ_BASE_URL . 'audio/audio-' . $id;
                $result .= '<audio class="' . $class . '">' .
                            '<source src="' . $base . '.mp3" type="audio/mpeg">' .
                            '<source src="' . $base . '.ogg" type="audio/ogg">' .
                            __( 'Your browser does not support this audio format.', 'ipquiz' ) .
                           '</audio>';
            }
            return $result;
        }
        
        /**
         * Get quiz info
         *
         * @param  object  $quiz
         * @return string
         */
        protected function _getQuizInfo( $quiz )
        {
            $result = '<div class="ipquiz-fb-info">' .
                        '<div class="ipquiz-fb-inner">' .
                            '<div class="ipquiz-info-header">' .
                                '<h1 class="ipquiz-info-title">' . $quiz->post_title . '</h1>' .
                            '</div>' .
                            '<div class="ipquiz-info-main ipquiz-clear">' .
                                '<div class="ipquiz-thumb">' .
                                    '<img src="" alt="">' .
                                '</div>' .
                                '<div class="ipquiz-info-content">' .
                                    $quiz->post_content .
                                '</div>' .
                            '</div>' .
                        '</div>' .
                      '</div>';
            return $result;
        }
        
        /**
         * Get quiz class
         *
         * @param  object  $quiz
         * @param  object   $setting
         * @return string
         */
        protected function _getQuizClass( $quiz, $setting )
        {
            $class  = 'ipquiz-item';
            $class .= ' ipquiz-design-' . $setting->getSetting( 'design', 'clean' );
            $class .= ' ipquiz-style-' . $setting->getSetting( 'style', 'grey' );
            return $class;
        }
        
        /**
         * Get quiz style
         *
         * @param  object  $quiz
         * @param  object   $setting
         * @return string
         */
        protected function _getQuizStyle( $quiz, $setting )
        {
            $style = '';
            return $style;
        }
        
        /**
         * Get quiz content
         *
         * @param  object  $quiz
         * @param  object  $setting
         * @param  boolean $intro
         * @return string
         */
        protected function _getQuizContent( $quiz, $setting, $intro = true )
        {
            if ( $intro ) {
                $result = $this->getIntro( $quiz, $setting );
            } else {
                $result = '';
            }
            return $result;
        }
        
        /**
         * Get quiz intro content
         *
         * @param  object  $quiz
         * @param  object  $setting
         * @return string
         */
        public function getIntro( $quiz, $setting )
        {
            $image = '';
            if ( $imgId = $setting->getSetting( 'image' ) ) {
                $imgId = absint( $imgId );
                if ( $attach = get_post( $imgId )) {
                    $image = '<div class="ipquiz-thumb"><img src="' . $attach->guid . '" alt=""></div>';
                }
            }
            $class = 'ipquiz-intro ipquiz-step';
            if ( empty( $image )) {
                $class .= ' ipquiz-noimage';
            }
            $result = '<div class="' . $class . '">' .
                        '<div class="ipquiz-step-inner">'.
                            '<div class="ipquiz-intro-content ipquiz-clear">'.
                                $image .
                                '<div class="ipquiz-description">' .
                                    $quiz->post_content .
                                '</div>' .
                            '</div>' .
                            '<div class="ipquiz-login-info">' .
                                '<p>' . $setting->getSetting( 'login_info', '' ) . '</p>' .
                                '<div class="ipquiz-fblogin">' .
                                    '<a class="ipquiz-btn-fblogin ipquiz-clear" href="#"><i class="fa fa-facebook"></i> <span class="ipquiz-btn-label"><em>Connect with</em> <strong>facebook</strong></span></a>' .
                                '</div>' .
                            '</div>' .
                        '</div>' .
                      '</div>';
            
            return $result;
        }
        
        /**
         * Get quiz questions
         *
         * @param  object  $quiz
         * @param  object  $user
         * @param  array   $questions
         * @param  object  $setting
         * @return string
         */
        public function getQuestions( $quiz, $user, $questions, $setting )
        {
            $count    = 0;
            $step     = 0;
            $qcount   = count( $questions );
            $choices  = $this->model->getChoices( $quiz->ID, $user->parc_id );
            $progress = array();
            $clast    = false;
            if ( $choices ) {
                $tempc = $choices;
                $clast = array_pop( $tempc );
                foreach( $choices as $choice ) {
                    $qtnId = $choice->qtn_id;
                    $progress[$qtnId] = $choice;
                }
            }
            $step   = count( $progress );
            if ( ! $clast || $clast->cho_status != '2' ) {
                $step++;
            }
            
            if ( $step > $qcount || ( $step == $qcount && ( $clast && $clast->cho_status == '2' ) ) )
            {
                $result = $this->getFinish( $quiz, $user, $questions, $setting );
                
                if ( $progress = $this->model->getProgress( $quiz->ID, $user->parc_id ) )
                {
                    if ( $progress->pro_status != 2 ) 
                    {
                        //Update point
                        $upoint  = $parc->point;
                        $upoint += $progress->pro_point;
                        $toUpdate = array( 'point' => $upoint );
                        $this->model->updateParticipant( $user->parc_id, $toUpdate );
                        
                        $toUpdate = array( 'pro_status' => 2 );
                        $this->model->updateProgress( $quiz->ID, $user->parc_id, $toUpdate );
                    }
                }
            } 
            else 
            {
                $result = '<div class="ipquiz-questions ipquiz-step">';
                if ( ! empty( $questions )) {
                    $order    = 1;
                    $nexton   = false;
                    $count    = count( $questions );
                    $ctimeout = false;
                    foreach( $questions as $question ) 
                    {
                        $cqtnId    = $question->qtn_id;
                        $isTimeout = false;
                        $isShow    = false;
                        if ( $clast && ( $question->qtn_id == $clast->qtn_id ) ) {
                            if ( $clast->cho_status == 2 ) {
                                $ctimeout = true;
                                $isShow = true;
                            } else {
                                $nexton = true; 
                            }
                        } elseif ( ! $clast && $order == 1 ) {
                            $isShow = true;
                        } elseif ( $nexton ) {
                            $isShow = true;
                            $nexton = false;
                        }
                        if ( isset( $progress[$cqtnId] ) ) {
                            $choice = $progress[$cqtnId];
                            if ( $choice->cho_status == 2 ) {
                                $isTimeout = true;
                            }
                        }
                        $result .= $this->_getQuestion( $question, $setting, $order, $isShow, $isTimeout );
                        $order++;
                    }
                }
                $result .= $this->_getProgress( $count, $setting, $step );
                $result .= '<div class="ipquiz-btn-next">' .
                                '<a href="#" class="ipquiz-btn">' . __( 'Next Question', 'ipquiz' ) . '</a>' . 
                            '</div>';
                $result .= $this->_getTimeCounter( $count, $setting, $ctimeout );
                $result .= '</div>';
            }
            return $result;
        }
        
        /**
         * Get quiz question
         *
         * @param  object  $question
         * @param  object  $setting
         * @param  int     $order
         * @param  boolean $isShow
         * @param  boolean $isTimeout
         * @return string
         */
        protected function _getQuestion( $question, $setting, $order, $isShow = true, $isTimeout = false )
        {
            if ( $isShow ) {
                $dsp = '';
            } else {
                $dsp = ' style="display: none;"';
            }
            $image = '';
            if ( ! empty( $question->qtn_image ) ) {
                $imgId = absint( $question->qtn_image );
                if ( $attach = get_post( $imgId )) {
                    $image = '<div class="ipquiz-thumb"><div class="ipquiz-thumb-deco"><img src="' . $attach->guid . '" alt=""></div></div>';
                }
            }
            $class  = 'ipquiz-question ipquiz-step-' . $order;
            $class .= ' ipquiz-question-' . $question->qtn_type;
            if ( empty( $image )) {
                $class .= ' ipquiz-noimage';
            }
            if ( $isTimeout ) {
                $class .= ' ipquiz-in-timeout';
            }
            $result = '<div data-id="' . $question->qtn_id . '" class="' . $class . '"' . $dsp . '>' .
                        '<div class="ipquiz-step-inner">' .
                            '<div class="ipquiz-question-header ipquiz-clear">' .
                                '<div class="ipquiz-nomor">' .
                                    '<strong>' . $order . '</strong>' .
                                '</div>' .
                                '<div class="ipquiz-qcontent">' .
                                    $question->qtn_content .
                                '</div>' .
                            '</div>' .
                            '<div class="ipquiz-question-main ipquiz-clear">' .
                                $this->_getAnswers( $question, $setting, $isTimeout ) .
                                $image .
                            '</div>' .
                        '</div>' .
                      '</div>';
            return $result;
        }
        
        /**
         * Get question progress
         *
         * @param  int     $count
         * @param  object  $setting
         * @param  int     $step
         * @return string
         */
        protected function _getProgress( $count, $setting, $step = 1 )
        {
            $count  = absint( $count );
            $width  = 100/$count * $step;
            $result =  '<div class="ipquiz-progress">' .
                            '<div class="ipquiz-meter">' .
                                '<span class="ipquiz-progress-val" style="width: ' . $width . '%;"></span>' .
                                '<strong class="ipquiz-progress-info" style="left: ' . $width . '%;">' . $step . '/' . $count . '</strong>' .
                            '</div>' .
                        '</div>';
            return $result;
        }
        
        /**
         * Get time counter
         *
         * @param  int     $count
         * @param  object  $setting
         * @param  boolean $isTimeout
         * @return string
         */
        protected function _getTimeCounter( $count, $setting, $isTimeout = false )
        {
            $result = '';
            if ( $time = $setting->getSetting( 'timelimit' ) )
            {
                $ttime = $time;
                if ( $isTimeout ) {
                    $width = '100';
                    $ttime = 0;
                } else {
                    $width = '0';
                }
                $result =  '<div class="ipquiz-time-counter">' .
                                '<div data-time="' . $time . '" class="ipquiz-counter">' .
                                    '<canvas data-color="#4aa4d6" data-size="' . $width . '" height="56" width="56" class="ipquiz-counter-meter"></canvas>' .
                                    '<canvas data-color="#27343c" data-size="100" height="56" width="56" class="ipquiz-counter-bg"></canvas>' .
                                    '<div class="ipquiz-counter-info">' . $ttime . '</div>' . 
                                '</div>' .
                            '</div>';
            }
            return $result;
        }
        
        /**
         * Get question answers
         *
         * @param  object  $question
         * @param  object  $setting
         * @param  boolean $isTimeout
         * @return string
         */
        protected function _getAnswers( $question, $setting, $isTimeout = false )
        {
            $result = '';
            if ( $answers = $this->model->getAnswers( $question->qtn_id ))
            {
                $dspInfo = '';
                if ( ! $isTimeout ) {
                    $dspInfo = ' style="display: none;"';
                }
                $result .= '<div class="ipquiz-answers">';
                $result .= '<ol>';
                foreach( $answers as $answer ) {
                    $result .= $this->_getAnswer( $answer, $question, $setting );
                }
                $result .= '</ol>';
                $result .= '<div class="ipquiz-timeout-info"' . $dspInfo . '>';
                $result .= '<p>' . $setting->getSetting( 'timeout_info', '' ) . '</p>';
                $result .= '</div>';
                $result .= '</div>';
            }
            return $result;
        }
        
        /**
         * Get question answer
         *
         * @param  object  $answer
         * @param  object  $question
         * @param  object  $setting
         * @return string
         */
        protected function _getAnswer( $answer, $question, $setting )
        {
            $key    = $question->qtn_id . '|' . $answer->ans_id . '|' . $answer->ans_status;
            $anskey = base64_encode( $key );
            $result = '<li data-id="' . $answer->ans_id . '" class="ipquiz-answer">' . 
                        '<div class="ipquiz-answer-inner">' .
                            '<label>' .
                                $this->_getAnswerCheck( $answer, $question, $setting ) .
                                '<strong class="ipquiz-answer-label">' .
                                    $answer->ans_content .
                                '</strong>' .
                            '</label>' .
                        '</div>' .
                        '<input class="ipquiz-answer-value" type="hidden" name="answer_value_' . $answer->ans_id . '" value="0">' .
                      '</li>';
            return $result;
        }
        
        /**
         * Get question answer check
         *
         * @param  object  $answer
         * @param  object  $question
         * @param  object  $setting
         * @return string
         */
        protected function _getAnswerCheck( $answer, $question, $setting )
        {
            if ( $question->qtn_type == 'multiple' )
            {
                $result = '<span class="ipquiz-answer-checkbox">' .
                               '<span class="ipquiz-checkbox"></span>' .
                               '<input class="ipquiz-answer-check" type="checkbox" name="answer_' . $question->qtn_id . '[]" value="' . $answer->ans_id . '" />' .
                               '<span class="ipquiz-checkbox"></span>' .
                           '</span>';
            } else {
                $result = '<span class="ipquiz-answer-radio">' .
                               '<span class="ipquiz-radio"></span>' .
                               '<input class="ipquiz-answer-check" type="radio" name="answer_' . $question->qtn_id . '" value="' . $answer->ans_id . '" />' .
                               '<span class="ipquiz-radio"></span>' .
                           '</span>';
            }
            return $result;
        }
        
        /**
         * Get quiz finish content
         *
         * @param  object  $quiz
         * @param  object  $participant
         * @param  array   $questions
         * @param  object  $setting
         * @return string
         */
        public function getFinish( $quiz, $participant, $questions, $setting )
        {
            $result    = '';
            if ( ! empty( $questions )) 
            {
                $count = count( $questions );
                $image = '';
                if ( $imgId = $setting->getSetting( 'image' ) ) {
                    $imgId = absint( $imgId );
                    if ( $attach = get_post( $imgId )) {
                        $image = '<div class="ipquiz-thumb"><div class="ipquiz-thumb-deco"><img src="' . $attach->guid . '" alt=""></div></div>';
                    }
                }
                $count++;
                $redir = '';
                $class = 'ipquiz-finish ipquiz-step';
                if ( empty( $image )) {
                    $class .= ' ipquiz-noimage';
                }
                if ( $setting->getSetting( 'redirect_on', 'false' ) == 'true' ) 
                {
                    $tout   = $setting->getSetting( 'redirect_timeout' );
                    $url    = $setting->getSetting( 'redirect_url', '#' );
                    $class .= ' ipquiz-redirect-on';
                    $redir  = '<div id="ipquiz-redirect-sett" data-timeout="' . $tout . '" data-url="' . $url . '"></div>';
                }
                $cinfo  = $setting->getSetting( 'congrats_info', '' );
                $pinfo  = $this->_getPointInfo( $quiz, $participant, $questions, $setting );
                $result = '<div class="' . $class . '">' .
                            $redir . 
                            '<div class="ipquiz-step-inner">'.
                                '<div class="ipquiz-finish-content">'.
                                    '<div class="ipquiz-clear">'.
                                        $image .
                                        '<div class="ipquiz-finish-info">' .
                                            '<div class="ipquiz-info-point">' . $pinfo . '</div>' .
                                            '<div class="ipquiz-info-congrats">' . $cinfo . '</div>' .
                                            $this->_getFinishButton( $setting ) .
                                        '</div>' .
                                    '</div>' .
                                '</div>' .
                            '</div>' .
                          '</div>';
            }
            return $result;
        }
        
        /**
         * Get point info
         *
         * @param  object  $quiz
         * @param  object  $participant
         * @param  array   $questions
         * @param  object  $setting
         * @return string
         */
        protected function _getPointInfo( $quiz, $participant, $questions, $setting )
        {
            $pinfo   = $setting->getSetting( 'point_info', '' );
            $plus    = $setting->getSetting( 'correct_point', 1 );
            $minus   = $setting->getSetting( 'wrong_point', 0 );
            
            $choices = $this->model->getChoices( $quiz->ID, $participant->parc_id );
            $count   = count( $questions );
            $point   = 0;
            $cpoint  = 0;
            $wpoint  = 0;
            if ( ! empty( $choices )) {
                foreach( $choices as $choice ) {
                    if ( $choice->cho_status == 1 ) {
                        $point += $plus;
                        $cpoint++;
                    } else {
                        $point -= $minus;
                        $wpoint++;
                    }
                }
            }
            $keys  = array(
                '%username%',
                '%count%',
                '%correct%',
                '%wrong%',
                '%point%'
            );
            $data = array(
                $participant->name,
                $count,
                $cpoint,
                $wpoint,
                $point
            );
            $pinfo = str_replace( $keys, $data, $pinfo );
            return $pinfo;
        }
        
        /**
         * Get quiz finish content
         *
         * @param  object  $setting
         * @return string
         */
        protected function _getFinishButton( $setting )
        {
            $result  = '<div class="ipquiz-finish-buttons">';
            $result .= '<div class="ipquiz-buttons">';
            
            if ( $setting->getSetting( 'button_share', 'true' ) == 'true' )
            {
                $result .= '<div class="ipquiz-finish-btn">' . 
                                '<a href="#" class="ipquiz-btn-share ipquiz-clear">' .
                                    '<i class="fa fa-facebook"></i>' .
                                    '<span class="ipquiz-btn-label">' . __( 'Share on timeline', 'ipquiz' ) . '</span>' .
                                '</a>' .
                            '</div>';
            }
            if ( $setting->getSetting( 'button_invite', 'true' ) == 'true' )
            {
                $result .= '<div class="ipquiz-finish-btn">' . 
                                '<a href="#" class="ipquiz-btn-invite ipquiz-clear">' .
                                    '<i class="fa fa-envelope"></i>' .
                                    '<span class="ipquiz-btn-label">' . __( 'Invite Friends', 'ipquiz' ) . '</span>' .
                                '</a>' .
                            '</div>';
            }
            if ( $setting->getSetting( 'button_topscore', 'true' ) == 'true' )
            {
                $result .= '<div class="ipquiz-finish-btn">' . 
                                '<a href="#" class="ipquiz-btn-topscore ipquiz-clear">' .
                                    '<i class="fa fa-trophy"></i>' .
                                    '<span class="ipquiz-btn-label">' . __( 'Top Scores', 'ipquiz' ) . '</span>' .
                                '</a>' .
                            '</div>';
            }
            
            $result .= '</div>';
            $result .= '</div>';
            return $result;
        }
    }
}