<?php
if ( ! class_exists( 'IPQuiz_Widget' )) {

    /**
     * Class for handling display widget
     */
    class IPQuiz_Widget extends WP_Widget
    {
        /**
         * Register widget with WordPress.
         */
        public function __construct() 
        {
            parent::__construct(
                'ipquiz_widget', // Base ID
                'Quiz Promo', // Name
                array( 'description' => __( 'A Widget for display quiz promo', 'ipquiz' ), )
            );
        }

        /**
         * Front-end display of widget.
         *
         * @see WP_Widget::widget()
         *
         * @param array $args     Widget arguments.
         * @param array $instance Saved values from database.
         */
        public function widget( $args, $instance ) 
        {
            $factory = new IPQuiz_FactoryPromo();
            $result  = $factory->widget( $args, $instance );
            echo $result;
        }

        /**
         * Back-end widget form.
         *
         * @see WP_Widget::form()
         *
         * @param array $instance Previously saved values from database.
         */
        public function form( $instance ) 
        {
            $promoId = 0;
            if ( isset( $instance[ 'promo' ] ) ) {
                $promoId = $instance[ 'promo' ];
            }
            $args = array(
                'orderby'        => 'post_date',
                'order'          => 'DESC', 
                'post_type'      => 'ipquiz-promo-item',
                'post_status'    => array('publish')
            );
            $query  = new WP_Query;
            $promos = $query->query( $args );
            ?>
            <br />
            <div>
                <label><strong><?php _e( 'Promo', 'ipquiz' ); ?></strong></label> 
                <div class="ipadm-input-wrap">
                    <select name="<?php echo $this->get_field_name( 'promo' ); ?>">
                        <?php
                        foreach( $promos as $promo ) 
                        {
                            $selected = '';
                            if ( $promoId == $promo->ID ) {
                                $selected = ' selected="selected"';
                            }
                            echo '<option' . $selected . ' value="' . $promo->ID .  '">' . $promo->post_title . '</option>';
                        }
                        ?>
                    </select>
                </div>
            </div>
            <br />
            <br />
            <?php 
        }

        /**
         * Sanitize widget form values as they are saved.
         *
         * @see WP_Widget::update()
         *
         * @param array $new_instance Values just sent to be saved.
         * @param array $old_instance Previously saved values from database.
         *
         * @return array Updated safe values to be saved.
         */
        public function update( $new_instance, $old_instance ) 
        {
            $promo = '';
            if ( ! empty( $new_instance['promo'] )) {
                $promo = absint( $new_instance['promo'] );
            } elseif ( ! empty( $old_instance['promo'] ) ) {
                $promo = absint( $old_instance['promo'] );
            }
            $instance = array(
                'promo'  => $promo
            );
            return $instance;
        }
    }

}
?>